-- main.lua for rope.love

function love.load()
	gr, lk			= love.graphics, love.keyboard
	width, height	= gr.getWidth(), gr.getHeight()
	ball				= gr.newImage('ball.png')
	gr.setBackgroundColor( 0, 0, 25, 255 )
	gr.setLineWidth( 3 )
	love.window.setTitle( "micha's rope trick" )
	angle				= 0
	gravity			= 2000
	length			= 20
	ropeSpeed		= 15
	speed				= 5
	damp				= 0.999
	node				= {}
	node.n			= 20
	nIterations 	= 10
	for i = 1,node.n do
		node[i] = {
			x		= 400 + 5 * i,
			y		= 50  + 5 * i,
			xold	= 400 + 5 * i,
			yold	=  50 + 5 * i,
			}
		end
	end

function love.update()
	dt = 0.003
	-- Advance each node by velocity
	for i = 1, node.n do
		node[i].x, node[i].xold = (1+damp) * node[i].x - damp * node[i].xold, node[i].x
		node[i].y, node[i].yold = (1+damp) * node[i].y - damp * node[i].yold, node[i].y
		end
	node[1].x = 400
	node[1].y = 50
	node[#node].y = node[#node].y + gravity * dt * dt
	if lk.isDown("right")then node[#node].x = node[#node].x + speed*dt end
	if lk.isDown("left")	then node[#node].x = node[#node].x - speed*dt end
	if lk.isDown("up")	then length	= math.max( 0, length - ropeSpeed * dt ) end
	if lk.isDown("down")	then length	= length + ropeSpeed * dt	end
	-- Force rope constraints
	for j = 1, nIterations do
	-- Iterate over inner rope parts, keeping ends at constant positions
		for i = 1, node.n - 2 do
			local dx = node[i+1].x - node[i].x
			local dy = node[i+1].y - node[i].y
			local dist = math.sqrt( dx*dx + dy*dy )
			if dist ~= 0 then
				-- Force constraint
				node[i+1].x = node[i].x + length * dx / dist
				node[i+1].y = node[i].y + length * dy / dist
				end
			end
		for i = node.n - 1, 2, -1 do
			local dx = node[i].x - node[i+1].x
			local dy = node[i].y - node[i+1].y
			local dist = math.sqrt( dx*dx + dy*dy )
			if dist ~= 0 then
				-- Force constraint
				node[i].x = node[i+1].x + length * dx / dist
				node[i].y = node[i+1].y + length * dy / dist
				end
			end
		end
	-- One final sweep, allowing to move the free end
	for i = 1, node.n - 1 do
		dx = node[i+1].x - node[i].x
		dy = node[i+1].y - node[i].y
		dist = math.sqrt( dx*dx + dy*dy )
		if dist ~= 0 then
			-- Force constraint
			node[i+1].x = node[i].x + length * dx / dist
			node[i+1].y = node[i].y + length * dy / dist
			end
		end
	dy = node[node.n].y - node[node.n-1].y
	dx = node[node.n].x - node[node.n-1].x
	angle = math.atan2( -dx, dy )
	end

function love.draw()
	gr.setColor(55,55,55,255)
	for i = 1, node.n-1 do
		gr.line( node[i].x, node[i].y, node[i+1].x, node[i+1].y )
		end
	gr.setColor(255,255,0,255)
	gr.draw( ball, node[node.n].x, node[node.n].y, angle, 1, 1, 8, 4 )
	gr.polygon('fill',node[1].x,node[1].y,node[1].x+10,node[1].y-10,node[1].x-10,node[1].y-10)
	if data then
		for i = 1, #node do
			msg = math.floor( node[i].x )..', '..math.floor( node[i].y )
			gr.print( msg, 10, 12*i )
			end
		end
	gr.print( 'fps: '..love.timer.getFPS( ), 10, height -20 )
	end

function love.keypressed(key)
	if key == 'escape' then love.event.push( 'quit' ) end
	if key == 'i' then data = not data end
	if key == 'w' then node[#node].y = node[#node].y-2 end
	if key == 's' then node[#node].y = node[#node].y+2 end
	if key == 'a' then node[#node].x = node[#node].x-2 end
	if key == 'd' then node[#node].x = node[#node].x+2 end
	end
